$versionMap = {}
VERSIONING = 'versioning'

def copy_output_files fromDir, filePattern, outDir
  FileUtils.mkdir_p outDir unless exists?(outDir)
  Dir.glob(File.join(fromDir, filePattern)){|file|
    copy(file, outDir) if File.file?(file)
  }
end

def copy_full_directory fromDir, outDir
   FileUtils.cp_r(File.join(fromDir, "."), outDir)
end


def project_outputs props
  props[:projects].map{ |p| "src/#{p}/bin/#{BUILD_CONFIG}/#{p}.dll" }.
    concat( props[:projects].map{ |p| "src/#{p}/bin/#{BUILD_CONFIG}/#{p}.exe" } ).
    find_all{ |path| exists?(path) }
end

def get_commit_hash_and_date
  begin
    commit = `git log -1 --pretty=format:%H`
    git_date = `git log -1 --date=iso --pretty=format:%ad`
    commit_date = DateTime.parse( git_date ).strftime("%Y-%m-%d %H%M%S")
  rescue
    commit = "git unavailable"
  end

  [commit, commit_date]
end

def add_files stage, what_dlls, nuspec, folder='lib'
  [['net35', 'net-3.5'], ['net40', 'net-4.0'], ['net45', 'net-4.5']].each{|fw|
    takeFrom = File.join(stage, fw[1], what_dlls)
    Dir.glob(takeFrom).each do |f|
      nuspec.file(f.gsub("/", "\\"), "#{folder}\\#{fw[0]}")
    end
  }
end

def commit_data
  begin
    commit = `git rev-parse --short HEAD`.chomp()[0,6]
    git_date = `git log -1 --date=iso --pretty=format:%ad`
    commit_date = DateTime.parse( git_date ).strftime("%Y-%m-%d %H%M%S")
  rescue Exception => e
    puts e.inspect
    commit = (ENV['BUILD_VCS_NUMBER'] || "000000")[0,6]
    commit_date = Time.new.strftime("%Y-%m-%d %H%M%S")
  end
  [commit, commit_date]
end

def waitfor &block
  checks = 0

  until block.call || checks >10
    sleep 0.5
    checks += 1
  end

  raise 'Waitfor timeout expired. Make sure that you aren\'t running something from the build output folders, or that you have browsed to it through Explorer.' if checks > 10
end

def cleantask props

  if props.has_key?(:output) && File.directory?(props[:output])
    FileUtils.rm_rf props[:output]
    waitfor { !exists?(props[:output]) }
  end

  if props.has_key?(:artifacts) && File.directory?(props[:artifacts])
    FileUtils.rm_rf props[:artifacts]
    waitfor { !exists?(props[:artifacts]) }
  end

  if props.has_key?(:output)
    Dir.mkdir props[:output]
  end
  if props.has_key?(:artifacts)
    Dir.mkdir props[:artifacts]
  end
end

def restore_nuget nuget_exe
  sh 'powershell', '-Command', "& { Invoke-RestMethod https://www.nuget.org/nuget.exe -OutFile '#{Dir.pwd}/#{nuget_exe}' }"
  sh "#{Dir.pwd}/#{nuget_exe}", 'restore', "src/#{PRODUCT}.sln"
end

def restore_project_pkgs proj
  msbuild :nuget_restore do |msb|
    msb.use :net4
    msb.targets :RestorePackages
    msb.solution = proj
  end
end

def clean_build msb, solution
  msb.properties :Configuration => 'Release',
    :Platform => 'Any CPU',
    :VisualStudioVersion => '12.0'
  msb.use :net4
  msb.solution = solution
end

def versioning project=nil, file=nil
  if !project.nil? && !file.nil?
    v = SemVer.new
    path = File.join VERSIONING, file
    v.load path
    ver = v
    commitData = commit_data()
    # nuget (not full semver 2.0.0-rc.1 support) see http://nuget.codeplex.com/workitem/1796
    ENV["#{project}_NUGET_VERSION"] = ver.format('%M.%m.%p%s')

    # extensible number w/ git hash
    ENV["#{project}_BUILD_VERSION"] =  ver.format('%M.%m.%p%s') + ".#{commitData[0]}"

    # purely M.m.p format
    sem =  SemVer.new(ver.major, ver.minor,  ver.patch).format '%M.%m.%p'
    build_counter = (ENV['BUILD_COUNTER'] || '0').to_i
    ENV["#{project}_FORMAL_VERSION"] = "#{sem}.#{build_counter}"
  else
    ver = SemVer.find

    commitData = commit_data()

    build_counter = (ENV['BUILD_COUNTER'] || '0').to_i

    # extensible number w/ git hash
    ENV['BUILD_VERSION'] = $versionMap[:build_version] = ver.format('%M.%m.%p%s') + ".#{commitData[0]}"

    # nuget (not full semver 2.0.0-rc.1 support) see http://nuget.codeplex.com/workitem/1796
    ENV['NUGET_VERSION'] = $versionMap[:nuget_version] = ver.format('%M.%m.%p%s')

    ENV['PLATFORM_VERSION'] = $versionMap[:platform_version] = Time.new.strftime('%y.%-m.%-d') + ".#{build_counter}"

    ENV['PLATFORM_BUILD_VERSION'] = $versionMap[:platform_build_version] = Time.new.strftime('%y.%-m.%-d') + ".#{commitData[0]}"

    sem =  SemVer.new(ver.major, ver.minor,  ver.patch).format '%M.%m.%p'

    ENV['FORMAL_VERSION'] = $versionMap[:formal_version] = "#{sem}.#{build_counter}"

    teamcity_build_number
  end
end

# Tell teamcity our decision to set the build.number environment variable
def teamcity_build_number
  platform_version  = Time.new.strftime('%y.%-m.%-d') + ".#{(ENV['BUILD_NUMBER'] || '0')}"
  puts "##teamcity[buildNumber '#{platform_version}']"
end

def set_framework_version asm
  set_version asm, $versionMap[:formal_version], $versionMap[:formal_version], $versionMap[:build_version], 'Framework'
end

def set_solution_version asm
  set_version asm, $versionMap[:platform_version], $versionMap[:platform_version], $versionMap[:platform_build_version], 'Solution'
end

def set_version asm, version, file_version, assembly_version, output_file, output_file_src='src'
  # Assembly file config
  asm.product_name = PRODUCT
  asm.description = PRODUCT_DESCRIPTION
  asm.version = version
  asm.file_version = file_version
  asm.custom_attributes :AssemblyInformationalVersion => assembly_version,
    :ComVisibleAttribute => false,
    :CLSCompliantAttribute => true
  asm.copyright = COPYRIGHT
  asm.output_file = "#{output_file_src}/#{output_file}.Version.cs"
  asm.namespaces 'System', 'System.Reflection', 'System.Runtime.InteropServices'
end

# checks nuget dependency
def check_package_version_dependency package_uri, package, version='IsLatestVersion'
  # retrieve package version info
  response = Net::HTTP.get_response(URI.parse(package_uri.sub('pkg', package))) # get_response takes a URI object
  package_info = response.body
  xml = Nokogiri::XML package_info
  xml.xpath("//m:properties/d:#{version}").each { |e|
    if e.text.to_s == 'true'
      version = e.parent.xpath('d:Version').text
    end
  }

  # grab all packages.config files
  config_files = Dir.glob '**/packages.config'

  # for each file match version. Return false is check fails
  config_files.each{ |file|
    doc = Nokogiri::XML(File.read file)
    node = doc.at_xpath("//*[@id=\"#{package}\"]")
    if !node.nil?
      config_version = node.attr('version')
      puts "Package: #{package} Latest Version: #{version} File: #{file} File package version: #{config_version}"
      return false unless config_version.to_s == version
    end
  }

  return true
end

def copy_runtime_artifacts package

  # grab all packages.config files
  config_files = Dir.glob '**/packages.config'
  config_version = ''

  # find package version
  config_files.each{ |file|
    doc = Nokogiri::XML(File.read file)
    node = doc.at_xpath("//*[@id=\"#{package}\"]")
    if !node.nil?
      config_version = node.attr('version').to_s
      break
    end
  }
  puts config_version

  # copy artifacts
  source = "#{ENV['RuntimePath']}/#{config_version}"
  dest = File.join(File.expand_path('.'), 'RuntimeService')
  puts source
  puts dest
  copy_full_directory source, dest
end

# Gathers all rake files and runs the task list
def package rake_files_path, task_list
  if task_list.nil?
    puts "No task to execute"
  else
    Dir.glob("#{rake_files_path}/*.rb").each {
      |r| load r
    }
    task_list.each { |eachtask|
      Rake::Task["#{eachtask}"].invoke
    }
  end
end
